package thinkAlike;

import java.io.IOException;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Item;
import javax.microedition.media.*;
import javax.microedition.media.control.*;
import net.sourceforge.jxa.*;

/**
 * This class handels all the logic of the application.
 * It also implements the XMPPListener interface for handeling actions that happend on the XMPP connection.
 * 
 * It is created by the a ScreenControl object, that is the startingpoint of the MIDlet.
 * @author runar
 * @version 1.0
 */
public class ProgramControl implements XmppListener {
    
    private ThinkRequest thinkRequest;
    private Jxa jxa;
    private ScreenControl sc;
    private Player mPlayer;
    private VideoControl mVideoControl;
    
    private String host = "horus.nta.no";
    private String port = "5222";
    private String username = "test";
    private String password = "passord";
    private String resource = "thinkalike";
    private int priority = 5;
    
    private String replyMessage;
    private String chatlog = ""; //
    private byte[] rawImage = null;
    private Image image = null;
    
    /**
     * Initiates the ProgramControl object to handle the logic of the application
     * @param sc a pointer to the ScreenControl object that handeles all the interaction with the display
     */
    public ProgramControl(ScreenControl sc) {
        this.sc = sc;
    }
    
    /**
     * Generates a new Jxa object that automatically connects to an XMPP server.
     * @return always true, can be used to reply if connection fails, but as implementation is now this is not necessary
     * @throws java.io.IOException if initial connection fails
     */
    public boolean connect() throws IOException{
        jxa = new Jxa(host,port,username,password,resource,priority);
        jxa.addListener(this);
        return true;
    }
    
    /**
     * Used to send a simple message with the Jxa object
     * @param to the recipient of the message
     * @param message the message string
     * @return not in use, can be used to return the outcome of sending the message.
     */
    public boolean sendMessage(String to, String message){
        jxa.sendMessage(to, message);
        return false;
    }
    
    /**
     * Sets the status of the XMPP connection using the Jxa object
     * @param show string explaining status
     * @param status actual status (available/unavailable)
     * @param priority default 5, not elsewise in use
     * @return not in use, can be used to respond to result of trying to set the status
     */
    public boolean setStatus(String show, String status, int priority){
        jxa.setStatus(show,status,5);
        return false;
    }
    
    /**
     * Setting the login parameters of the XMPP session that will later be initiated
     * @param host the XMPP host name
     * @param port the port in use, most common 5222 (if secure connection: 5223)
     * @param username the username of the actor, username@xmpphostdomain
     * @param password password in clear text, this is one of the weakest security points of the application
     * @param resource the resource parameter of the XMPP client, in this case, always set to /thinkalike
     */
    public void setLoginParameters(String host, String port, String username, String password, String resource){
        this.host = host;
        this.port = port;
        this.username = username;
        this.password = password;
        this.resource = resource;
    }
    
    /*
     * This method gives classes like ThinkRequest access to the ScreenControl class
     */
    /**
     * Returns the pointer for the ScreenControl object that created this object
     * @return pointer to ScreenControl object
     */
    public ScreenControl getSc(){
        return sc;
    }
    
    /**
     * Returns the Jxa object that handels the XMPP connection going on.
     * @return pointer to the Jxa object
     */
    public Jxa getJxa(){
        return jxa;
    }
    
    /**
     * Returns the ThinkRequest object on going at the moment. 
     * 
     * Should be changed if one wants more than one ThinkRequest session going, this is not possible at the moment.
     * @return the ongoing ThinkRequest session
     */
    public ThinkRequest getThinkRequest(){
        return thinkRequest;
    }
    
    /**
     * Run at the end of a session, clears the chat log and resets the thinkrequest variable.
     */
    public void clearThinkAlikeSession(){
        thinkRequest = null;
        chatlog = "";
    }
    
    /**
     * The method creating the ThinkRequest, doing thinkgs a bit different depending on bein the initiator or the recipient.
     * 
     * If it is the recipient it also converts the incoming string representing the picture to a raw image.
     * @param to address of the recipient
     * @param from address of the initiator
     * @param pic picture represented by a text string
     * @param thinkword the word that is the word
     */
    public void makeThinkRequest(String to, String from, String pic, String thinkword){
        if(from == null){
            from = this.username;
        }
        if(pic == null){ // when Requester
            String tempImageString = Base64.encode(rawImage);
            thinkRequest = new ThinkRequest(this, to, from, tempImageString, thinkword);
        } else{ // when Reciever
            rawImage = Base64.decode(pic);
            image = Image.createImage(rawImage, 0, rawImage.length);
            thinkRequest = new ThinkRequest(this, username, from, pic, thinkword);
        }
    }
    
    /**
     * Initiated when in EndForm, sending the text inputted in the text box to the other part of the session-
     */
    public void sendChatMessage(){
        if(thinkRequest.isInitiator()){
            jxa.sendMessage(thinkRequest.getTo(),sc.get_chatField().getString());
        } else {
            jxa.sendMessage(thinkRequest.getFrom(),sc.get_chatField().getString());
        }
        chatlog += username + ": " + sc.get_chatField().getString() + "\n";
        sc.get_chatField().setString("");
        sc.get_chatString().setText(chatlog);
    }
    
    /**
     * Initializes the camera with it's related media manager.
     */
    
    public void initilizaCamera() {
        try {
            mPlayer = Manager.createPlayer("capture://video");
            mPlayer.realize();
            mPlayer.start();
        } catch (MediaException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        mVideoControl = (VideoControl) mPlayer.getControl("VideoControl");
    }
    
    /**
     * Returns the actual item containg the feed from the camera to put on the display
     * @return Camera item
     */
    public Item getCameraItem() {
        Item item = (Item)mVideoControl.initDisplayMode(GUIControl.USE_GUI_PRIMITIVE, null);
        return item;
    }
    
    /**
     * Handels the actual captering from the video source. Then resets the camera.
     */
    public void captureImage() {
        try {
            // Get the image.
            rawImage = mVideoControl.getSnapshot(null);
            image = Image.createImage(rawImage, 0, rawImage.length);
            mPlayer.close();
            mPlayer = null;
            mVideoControl = null;
        } catch (MediaException me){
            java.lang.System.out.println("*error* capturing image, error message: " + me.toString());
        }
    }
    
    /**
     * Returns the image assosiated with ThinkRequest session
     * @return raw image
     */
    public Image getImage(){
        if(image != null){
            return image;
        } else{
            return null;
        }
    }
    
    /**
     * Gets the words inputed by the recipient of the ThinkRequest from the screen and put them in the thinkrequest object. 
     * This method is called when the send button is pushed
     */
    public void setWords(){
        thinkRequest.setWords(sc.get_thinkOfWord1().getString(),sc.get_thinkOfWord2().getString(),sc.get_thinkOfWord3().getString());
    }
    
    /**
     * XmppListener internface: initiated when connecting to the XMPP server fails
     */
    public void onConnFailed() {
//        java.lang.System.out.println("*debug* connection failed");
        replyMessage = "Connection failed";
    }
    
    /**
     * XmppListener internface: initiated when authorization is sucsessful
     */
    public void onAuth() {
//        java.lang.System.out.println("*debug* authorized");
    }
    
    /**
     * XmppListener internface: initiated when authorization fials
     * @param message error message related to the failiar
     */
    public void onAuthFailed(final String message) {
//        java.lang.System.out.println("*debug* authorization failed with message:" + message);
    }
    
    /**
     * XmppListener internface: initiated when a message is recieved
     * @param from the sender of the message
     * @param body message body
     */
    public void onMessageEvent(final String from, final String body) {
        chatlog += from + ": " + body + "\n";
        sc.get_chatString().setText(chatlog);
        
//        java.lang.System.out.println("*debug* message event from " + from + " saying " + body);
    }
    
    /**
     * see XmppListener internface
     * @param jid .
     */
    public void onContactRemoveEvent(final String jid) {
    }
    
    /**
     * see XmppListener internface
     * @param jid .
     * @param name .
     * @param group .
     * @param subscription .
     */
    public void onContactEvent(final String jid, final String name, final String group, final String subscription) {
    }
    
    /**
     * see XmppListener internface
     * @param jid .
     * @param show .
     * @param status .
     */
    public void onStatusEvent(final String jid, final String show, final String status) {
    }
    
    /**
     * see XmppListener internface:
     * @param jid .
     */
    public void onSubscribeEvent(final String jid) {
    }
    
    /**
     * see XmppListener internface:
     * @param jid .
     */
    public void onUnsubscribeEvent(final String jid) {
    }
    
    /**
     * A special method added to the XmppListener class handeling IQ messages related to the ThinkAlike application. 
     * 
     * Some of the parameters might be null depending on if the event is a request or response.
     * @param thinktype either request or response
     * @param from the sender of the request
     * @param pic text representation of the picture
     * @param thinkword the word
     * @param word1 .
     * @param word2 .
     * @param word3 .
     */
    public void onThinkEvent(final String thinktype, final String from, final String pic, final String thinkword, final String word1, final String word2, final String word3){
        if(thinktype.equals("thinkrequest"))
        {
            makeThinkRequest(this.username, from, pic, thinkword);
            thinkRequest.isInitiator(false);
            thinkRequest.confirmRequest();
            sc.get_fromUserString().setText(from);
            sc.getDisplay().setCurrent(sc.get_recievedChallengeForm());
        } 
        else if(thinktype.equals("thinkresponse"))
        {
            thinkRequest.recievedResponse(word1, word2, word3);
            thinkRequest.isInitiator(true);
        }
    }
    
    
}






